####################################################################
# (C) Copyright 2016 Hewlett Packard Enterprise Development LP. 
# @(#) Metrocluster P9000 Continuous Access Module Script.
# @(#) Product Name                :  HP Metrocluster P9000CA
# @(#) Product Version             :  B.12.00.51
# @(#) Patch Name                  : 
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Metrocluster.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

if [ -f /etc/redhat-release ]; then
  DISTRO="REDHAT"
  CMCLUSTER="/usr/local/cmcluster"
else
  DISTRO="SUSE"
  CMCLUSTER="/opt/cmcluster"
fi

SG_UTILS=${CMCLUSTER}/conf/scripts/mscripts/utils.sh

export SG_UTILS

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

#########################################
# Source Metrocluster  utility functions.
##########################################

#MC_UTILS=/usr/local/cmcluster/conf/scripts/dts/mc_util.sh
MC_UTILS=${CMCLUSTER}/conf/scripts/dts/xpca_util.sh

export MC_UTILS

if [[ -f ${MC_UTILS} ]]; then
    . $MC_UTILS
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package Metrocluster utility functions file: ${MC_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Metrocluster utility functions file: ${MC_UTILS}"
    exit 1
fi

#################################
# Source hpux internal functions.
#################################

#if [[ -f $SG_INTERNAL_FUNCS_FILE ]] ; then
#    . $SG_INTERNAL_FUNCS_FILE
#    if (( $? != 0 )) ; then
#        sg_log 0 "ERROR: Unable to source SG internal functions file: $SG_INTERNAL_FUNCS_FILE"
#        exit 1
#    fi
#else
#    sg_log 0 "ERROR: Unable to find SG internal functions file: $SG_INTERNAL_FUNCS_FILE"
#    exit 1
#fi
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

# Exporting cmviewcl command output.
cmviewcl_output_mc=`cmviewcl -vf line -s config`

###############################################################################
# This function invokes the metrocluster xpca binary
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - metrocluster xpca operation is successful
#         exit - 0 - metrocluster xpca operation is not successful
#
###############################################################################

function runmcxpca
{
	typeset exit_val;
	typeset ret;
	if [[ -f ${CMCLUSTER}/bin/DRCheckXPCADevGrp ]]
	then
	
 			DRCheckXPCADevGrp $1 0 0 "" ""
			exit_val=$?
	
	else

        print "ERROR: Cannot find the file: DRCheckXPCADevGrp file. Exiting."
	exit 1
	
	fi

	if [[ $exit_val -eq 3 ]] # Storage switched from read-only to read-write
	then
                        typeset wwids=""
                        wwids=`/usr/bin/pairdisplay -g $XPCA_DEVICE_GROUP -I$XPCA_HORCMINST -l -few  -CLI | awk '{print $20}' | grep -v "^LUN-WWN"`
                        for disk in ${wwids[@]}; do
				disk=`echo $disk | awk '{print tolower($0)}'`
                                ${CMCLUSTER}/bin/mcxpca/mcxpca_multipath_rescan.sh 3${disk}
                        done

            sleep 10 # sleeping for rescan to take effect

	    for vg in ${SG_VG[*]} 
		do
			typeset hostid=""
			
			hostid=$(vgs --noheadings -o +tags $vg | awk  '{print $8}')
			if [[ "$hostid" = *,* ]]; then
		        sg_log 0 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
		        sg_log 0 "There cannot be more than one tag."
			sg_log 0 "Remove the tags using the command:"
        		for i in ${hostid}; do
		            sg_log 0 "\"vgchange --deltag $i $vg\"."
		        done
		        return 1
  		        fi
	
			if [[ "$hostid" != "" ]] ; then
			#	cmexec $hostid vgchange -a n $vg
				vgchange --deltag $hostid $vg 
			fi	
		done
	    echo "export SFC_MC_RESCAN_REQUIRED=\"1\"" >$SG_SRC_FRM_CHILD
	    exit_val=0
	fi

	if [[ $exit_val -ne 0 ]]
	then
	    exit $exit_val
	fi
	return 0
}


#######################################################################
#
# function sg_start_xpca() is called by master control script
# during metrocluster xpca package startup.
#
#######################################################################


function sg_start_xpca
{
	typeset envfile;
	typeset count;
        typeset ret;
	typeset version=0;

	checkifmodused "dts/sc"
	ret=$?
	if [ $ret -eq 0 ]
	then 
		sg_log 0 "Site Controller module detected "
		exit 0
	fi
	
	sg_log 0 "Starting Metrocluster XPCA Package"

	find_module_version "dts/mcxpca"
	version=$?

	runmcxpca $SG_PACKAGE_NAME
	to_exit=0
}

#######################################################################
#
# function sg_stop_xpca() is called by master control script
# during metrocluster xpca package stop.
#
#######################################################################


function sg_stop_xpca
{
    sg_log 0 "Stopping Metrocluster XPCA Package"
    # no op
    to_exit=0
}


#######################################################################
#
# function xpca_env_gen() is called by sg_validate_xpca
# during metrocluster xpca configuration application.
#
#######################################################################
function xpca_env_gen
{
	typeset exists;
	typeset retval;

	if [[ "$1" = "internal" ]]; then
                env_file_location="${XPCA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_xpca.env.internal"
        else
                env_file_location="${XPCA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_xpca.env"
        fi


        echo "#!usr/bin/sh
AUTO_FENCEDATA_SPLIT=$XPCA_AUTO_FENCEDATA_SPLIT
AUTO_NONCURDATA=$XPCA_AUTO_NONCURDATA
AUTO_PSUEPSUS=$XPCA_AUTO_PSUEPSUS
AUTO_PSUSSSWS=$XPCA_AUTO_PSUSSSWS
AUTO_SVOLPFUS=$XPCA_AUTO_SVOLPFUS
AUTO_SVOLPSUE=$XPCA_AUTO_SVOLPSUE
AUTO_SVOLPSUS=$XPCA_AUTO_SVOLPSUS
DEVICE_GROUP=$XPCA_DEVICE_GROUP
FENCE=$XPCA_FENCE
HORCMINST=$XPCA_HORCMINST
HORCMPERM=$XPCA_HORCMPERM
HORCTIMEOUT=$XPCA_HORCTIMEOUT
MULTIPLE_PVOL_OR_SVOL_FRAMES_FOR_PKG=$XPCA_MULTIPLE_PVOL_OR_SVOL_FRAMES_FOR_PKG
WAITTIME=$XPCA_WAITTIME
PKGDIR=$XPCA_DTS_PKG_DIR
CLUSTER_TYPE=$XPCA_CLUSTER_TYPE
" > $env_file_location

retval=$?

chmod 755 $env_file_location

exists=`set | grep -c XPCA_MON_POLL_INTERVAL`
if [ $exists -gt 0  ]; then
echo "MON_POLL_INTERVAL=$XPCA_MON_POLL_INTERVAL" >> $env_file_location
fi

exists=`set | grep -c XPCA_MON_NOTIFICATION_FREQUENCY`
if [ $exists -gt 0  ]; then
echo "MON_NOTIFICATION_FREQUENCY=$XPCA_MON_NOTIFICATION_FREQUENCY" >> $env_file_location
fi

exists=`set | grep -c XPCA_MON_NOTIFICATION_EMAIL`
if [ $exists -gt 0  ]; then
echo "MON_NOTIFICATION_EMAIL=$XPCA_MON_NOTIFICATION_EMAIL" >> $env_file_location
fi

exists=`set | grep -c XPCA_MON_NOTIFICATION_SYSLOG`
if [ $exists -gt 0  ]; then
echo "MON_NOTIFICATION_SYSLOG=$XPCA_MON_NOTIFICATION_SYSLOG" >> $env_file_location
fi

exists=`set | grep -c XPCA_MON_NOTIFICATION_CONSOLE`
if [ $exists -gt 0  ]; then
echo "MON_NOTIFICATION_CONSOLE=$XPCA_MON_NOTIFICATION_CONSOLE" >> $env_file_location
fi

exists=`set | grep -c XPCA_AUTO_RESYNC`
if [ $exists -gt 0  ]; then
echo "AUTO_RESYNC=$XPCA_AUTO_RESYNC" >> $env_file_location
fi
return $retval

}

#######################################################################
#
# function sg_validate_xpca() is called by master control script
# during metrocluster xpca configuration validation.
#
#######################################################################

function sg_validate_xpca
{
	# no op
	typeset version=0;
	find_module_version "dts/mcxpca"
	version=$?

	validate_mc
	if [[ $? -eq 1 ]]; then
		to_exit=1
		exit $to_exit
	fi

        if [[ $version -ge 1 ]]; then
        check_cc_module
	if [[ $? -eq 1 ]]; then
		sg_major_version=`${CMCLUSTER}/bin/cmversion | awk -F'.' '{ print $2 }'`
		sg_minor_version=`${CMCLUSTER}//bin/cmversion | awk -F'.' '{ print $3 }'`
		if [[ ${sg_major_version} -ge 12 ]]; then
			sg_verify_xpca_pkg
			to_exit=$?
		elif [[ ${sg_major_version} -eq 11 && ${sg_minor_version} -ge 20 ]]; then
			sg_verify_xpca_pkg
			to_exit=$?
		fi
	fi
	fi


}

######################################################################
#
# function verification_after_pkgstart is called when "cmcheckconf -v"
# is executed.This will be ideally called when the pkg is already up
# running.
#
#######################################################################

function verification_after_pkgstart
{
	RMFILE=/etc/mcxpca/raidmgr
	HORCMINST=horcm$XPCA_HORCMINST
	ETC_SERVICES=/etc/services
	CAT=/bin/cat
	PS=/bin/ps
	GREP=/bin/grep
	PAIRDISPLAY=/usr/bin/pairdisplay
	FORCEFLAG=${XPCA_DTS_PKG_DIR}/FORCEFLAG
        typeset conf_status=0
        typeset HORCMD=0
	typeset not_up=0

        if [[ ($XPCA_HORCMINST -ge 0) && ($XPCA_HORCMINST -le 9) ]];then
          HORCMD=horcmd_0$XPCA_HORCMINST
        else
          HORCMD=horcmd_$XPCA_HORCMINST
        fi

        if [[ -e $RMFILE ]];then
          . $RMFILE

           #Check if Raid Manager is not set to auto start
           if [[ $START_RAIDMGR -eq 0 ]];then
              echo "WARNING:Raid Manager is not set to auto start"
           fi

           #check if horcm instance is not configured in the Raid Manager file
           found=`echo $RAIDMGR_INSTANCE | awk -v pat=$XPCA_HORCMINST \
                  '{num=split($0,a,","); for(i=1;i<=num;i++){if (a[i] == pat){print a[i]}}}'`
           if [[ -z "$found" ]];then
             echo "WARNING:The horcm instance $XPCA_HORCMINST is not configured in the file $RMFILE"
           fi
        fi

	#check if Raid Manager is not running
	$PS -ef | $GREP $HORCMD | $GREP -v $GREP >>/dev/null
	if [[ $? -ne 0 ]]; then
		echo "WARNING:Raid Manager is not running"
		not_up=1
	fi
	
	#Check if horcm instance is not configured in /etc/services
        conf_status=`$GREP "$HORCMINST " < $ETC_SERVICES`
        if [[ $? -eq 0 ]];then #if configured, check if it is commented
         echo $conf_status | $GREP "^#" >>/dev/null
         if [[ $? -eq 0 ]]; then
                echo "WARNING:Horcm instance $HORCMINST is not configured
                        in /etc/services"
         fi
        else
         echo "WARNING:Horcm instance $HORCMINST is not configured
                        in /etc/services"
        fi

	#check if force flag is present
	if [[  -f $FORCEFLAG ]];then
		echo "WARNING:The FORCEFLAG $FORCEFLAG, is present in the package directory"
	fi

	#check if replication is happening
        if [[ $not_up -ne 0 ]];then #let us check only if RM is running
         if [[ $found -ne 1 ]];then #let us check only if there is pairdisplay command
                $PAIRDISPLAY -g $XPCA_DEVICE_GROUP -I$XPCA_HORCMINST | $GREP "PAIR" >>/dev/null
           if [[ $? -ne 0 ]]; then
                echo "WARNING:The replication is not happening"
           fi
         fi
        fi

}

#
# function sg_verify_xpca_pkg is called by sg_validate_xpca()
# during metrocluster xpca configuration validation.
#
#######################################################################

function sg_verify_xpca_pkg
{

# CMGETPKGENV=/usr/sbin/cmgetpkgenv
 CMGETPKGENV=${CMCLUSTER}//bin/cmgetpkgenv

#Step 1 : check if the software is installed.
        output=`which pairdisplay 2>&1 >> /dev/null | grep -c "no pairdisplay"`
        if [[ $output -ne 0 ]]; then
	  found=1
          errors[$errcount]="Raid Manager is not installed. Refer to the HP StorageWorks XP/P9000 RAID Manager Installation and Configuration Guide for instructions to install this software"
          let errcount=$errcount+1
        fi


#Step 2 : Verify the values specified for Metrocluster
#        module parameters are valid. cmapplyconf will
#       anyway generate this file. The environment file
#       however needs to be generated during cmcheckconf -P/-p
#       with a temporary name to validate the inputs in the
#       package configuration file.

	typeset version=0;
	find_module_version "dts/mcxpca"
	version=$?
	
        internal_use=0
	xpca_env_gen "internal"
	to_exit=$?
	internal_use=1

        
        ${CMCLUSTER}/toolkit/SGCA/XPCAValidateAttrib.pl ${XPCA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_xpca.env.internal

        if [[ $? -ne 0 ]]; then
               errors[$errcount]="Validation of Metrocluster attributes failed."
               (( errcount=errcount+1 ))
        fi

        if [[ "${internal_use}" -eq 1 ]]; then
                rm -f ${XPCA_DTS_PKG_DIR}/${SG_PACKAGE_NAME}_xpca.env.internal
        fi
#Step 3 : VMFS verification
        
        if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
                verify_vmfs_details
                if [ $? -ne 0 ]; then
                    echo "ERROR: Failed to validate VMFS details mentioned in the package"
                    to_exit=1
                fi
        fi

#Step 4 : Verify if all disks used by a Metrocluster
#        package are replicated
#
        if [[ ${#SG_VMDK_FILE_NAME[*]} -le 0 ]]; then

            checklvmused ${SG_VG[*]}
            ret=$?
            if (( $ret == 0 )) || (( $ret == 2 )) ; then
	  	 list_of_vgs="${SG_VG[@]}"
                 lvm_disks_replicated
            fi

            i=0
            while [ $i -lt ${#errors[@]} ]; do
		echo ${errors[$i]}
                (( i = i + 1 ))
            done

            i=0
            while [ $i -lt ${#warnings[@]} ]; do
		echo ${warnings[$i]}
                (( i = i + 1 ))
            done
        fi
	sg_ver=`${CMCLUSTER}/bin/cmversion | cut -d "." -f 2-4`
	if [[ "$sg_ver" > "11.20.20" || "$sg_ver" = "11.20.20" ]]; then
            if [[ $SG_PRIV_CLUSTER_VERIFICATION == y ]];then
                #Let us do additional checks
                 verification_after_pkgstart
	    fi
	fi

        if  [[ ${errcount} -gt 0 ]]; then
                return 1
        else
                return 0
        fi
	
}

function verify_vmfs_details
{
        eligible_nodes_vmfs=""
        ro_nodes=""
        package_nodes=$(get_package_nodes)
        for node in ${package_nodes[*]}
        do
                  pairdisplay_result=$($SGSBIN/cmdo -n ${node} "/usr/bin/pairdisplay -g $XPCA_DEVICE_GROUP -I$XPCA_HORCMINST -l")
                  if [[ $? -ne 0 ]]; then
                      echo "Pairdisplay command failed on node ${node}."
                      return 1 
                  fi
                  echo "${pairdisplay_result}"|grep "P-VOL" > /dev/null
                  if [[ $? -eq 0 ]]; then
                      eligible_nodes_vmfs="$eligible_nodes_vmfs ${node}"
                  else
                      ro_nodes="$ro_nodes ${node}"
                  fi
        done

        eligible_nodes_vmfs=(${eligible_nodes_vmfs})
        ro_nodes=(${ro_nodes})

        # Pass the eligible nodelist to Master Control Script
        echo "export SFC_MC_READ_WRITE_NODELIST=\"${eligible_nodes_vmfs[*]}\"" >$SG_SRC_FRM_CHILD
        echo "export SFC_MC_READ_ONLY_NODELIST=\"${ro_nodes[*]}\"" >>$SG_SRC_FRM_CHILD


        cur_hostname=${SG_HOST_NAME%%.*}

        # Verification is done only from one node.
        if [[ ${cur_hostname} = ${eligible_nodes_vmfs[0]} ]]; then

             eligible_node_uuid=$(get_uuid_from_node_name ${eligible_nodes_vmfs[0]})
             ro_node_uuid=$(get_uuid_from_node_name ${ro_nodes[0]})

             # Getting VMFS details
             vmfs_details=$($SGSBIN/cmvmdiskquery  -n $eligible_node_uuid -n $ro_node_uuid )
             if [[ $? -ne 0 ]]; then
                     echo "ERROR: Unable to query the ESX Host or VCenter."
                     return 1
             fi
  
             # Verifying the RW site.
             typeset wwids_rw=""
             wwids_rw=`/usr/bin/pairdisplay -g $XPCA_DEVICE_GROUP -I$XPCA_HORCMINST -l -few  -CLI | awk '{print $20}' | grep -v "^LUN-WWN"`

             # Verifying that all the disks in the RCVG is part of the Package.
              for disk_wwid in ${wwids_rw}
                do
                disk_flag=0
                for ds_name in ${SG_DATASTORE_NAME[*]}
                    do
                         echo "$vmfs_details"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|grep -i "$disk_wwid" >/dev/null 2>&1
                         if [[ $? -eq 0 ]]; then
                             disk_flag=1
                         fi
                    done
                    if [[ $disk_flag -eq 0 ]];then
                          echo "WARNING: The disk with WWID $disk_wwid, configured as part of the Countinous Access Group $XPCA_DEVICE_GROUP"
                          echo "      is not part of any of the Datastore specified in the package."
                    fi
              done

              # Verifying that all the disk in the Data Store is part of RCVG
              disk_flag=0
              for ds_name in ${SG_DATASTORE_NAME[*]}
                do
                ds_wwids=`echo "${vmfs_details}"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|cut -d '=' -f2`
                #ds_wwids=`echo "${vmfs_details}"|grep "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn=naa."|cut -d '|' -f3|cut -d '.' -f2`
                for wwid in ${ds_wwids}
                    do
                    echo "${wwids_rw}"|grep -i "${wwid##*.}" >/dev/null 2>&1 
                    if [[ $? -ne 0 ]]; then
                         echo "ERROR: The disk with WWID ${wwid##*.}, used for  the Datastore ${ds_name} is not part of the Countinous Access Group $XPCA_DEVICE_GROUP"
                         disk_flag=1
                    fi
                done
              done
              if [[ $disk_flag -eq 1 ]];then
                   return 1
              fi

              # Verifying the RO site.
              typeset wwids_ro=""
              wwids_ro=`$SGSBIN/cmdo -n ${ro_nodes[0]} /usr/bin/pairdisplay -g $XPCA_DEVICE_GROUP -I$XPCA_HORCMINST -l -few  -CLI | awk '{print $20}' | grep -v "^LUN-WWN"`

              disk_flag=0
              # Verifying that all the disks in the RCVG is part of the Package.
              for disk_wwid in ${wwids_ro}
              do
                  echo "$vmfs_details"|grep -i "VirtualMachine:${ro_node_uuid}|Disk:" |grep -i $disk_wwid >/dev/null 2>&1
                  if [[ $? -ne 0 ]]; then
                        echo "WARNING: The disk with WWID $disk_wwid, configured as part of the Countinous Access Group $XPCA_DEVICE_GROUP"
                        echo "         in the RO site is not available to the host. Package will not start during fail-over to standby RO site."
                  fi
              done
        fi

        return 0
}
################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#


typeset to_exit=0;



case ${1} in
       start)      sg_start_xpca;;
        stop)      sg_stop_xpca;;
    validate)    sg_validate_xpca ;;
esac
#
# Exit to indicate success/failure to the master_control_script.sh
#

exit $to_exit


