####################################################################
# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP. 
# @(#) Metrocluster P9000 Continuous Access Module Script.
# @(#) Product Name                :  HP Metrocluster P9000CA
# @(#) Product Version             :  B.12.00.50
# @(#) Patch Name                  :
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Metrocluster.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

if [ -f /etc/redhat-release ]; then 
DISTRO="REDHAT"
grep 5 /etc/redhat-release 2>&1 > /dev/nul
  if [[ $? -eq 0 ]];then
        #In RH5 the path of pvdisplay is different 
	PVDISPLAY=/usr/sbin/pvdisplay
  else
	PVDISPLAY=/sbin/pvdisplay
  fi	
else
  DISTRO="SUSE"
  PVDISPLAY=/sbin/pvdisplay
fi

if [[ $DISTRO == "REDHAT" ]]; then 
SG_UTILS=/usr/local/cmcluster/conf/scripts/mscripts/utils.sh
elif [[ $DISTRO == "SUSE" ]]; then 
SG_UTILS=/opt/cmcluster/conf/scripts/mscripts/utils.sh
fi

export SG_UTILS

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
   . ${SG_UTILS}
   if (( $? != 0 ))
   then
      echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
      exit 1
   fi
else
   echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
   exit 1
fi

#typeset -A errors
#typeset -A warnings
#typeset -A list_of_dgs
#typeset -A list_of_vgs

errcount=0
warncount=0
internal_use=0

###############################################################################
# This function checks whether the volume manager is lvm or slvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - lvm is used
#          1 - lvm is not used
#          2 - slvm is used
#
###############################################################################

function checklvmused
{
   if [[ ${SG_VG[*]} != "" ]]
   then
      
      if [[ ${SG_VGCHANGE_CMD} = "" ]]
      then
         sg_log 0 "\tERROR: Function checklvmused"
         sg_log 0 "\tERROR: LVM Activation Command can not be empty while using LVM group"
         exit 1
      fi
      
      if [[ "${SG_VGCHANGE_CMD#*-a}" = " s" ]]
      then
         return 2;
      fi
      
      return 0
   fi
   return 1
}



###############################################################################
# This function checks whether given module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - module is used
#          1 - module is not used
#
###############################################################################

function checkifmodused
{
   typeset used_module=$1;
   let exit_val=1
   typeset module_name;
   typeset module;
   
   module_name=`echo "${SG_MODULE_NAME[@]}"`
   
   for module in $module_name
   do
      if [ "$module" = "$used_module" ]
      then
         let exit_val=0
         break
      fi
   done
   
   return $exit_val
   
}

###############################################################################
# This function returns the version of the given module
# Internal Use only
#
#
# Arguments: The name of the module
#
#
#
# Returns: The version of the module
#
###############################################################################

function find_module_version
{
   typeset used_module=$1;
   typeset module_name;
   let counter=0
   typeset module;
   typeset module_location;
   module_name=`echo "${SG_MODULE_NAME[@]}"`
   
   for module in $module_name
   do
      if [ "$module" = "$used_module" ]
      then
         module_location=$counter
         break
      else
         let counter=counter+1
      fi
   done
   return ${SG_MODULE_VERSION[${module_location}]}
   
   
}


function validate_mc
{

   typeset envfile;
   typeset samefile;
   typeset count;

   sg_log 0 "Validating Metrocluster Configuration"

   let version=0

   find_module_version "dts/xpca"
   version=$?

      if [[ ! -d $XPCA_DTS_PKG_DIR ]]; then
         mkdir -p -m 755 $XPCA_DTS_PKG_DIR
         if [ $? -eq 0 ]; then
            sg_log 0 "Creation of \"$XPCA_DTS_PKG_DIR\" successful"
         else
            echo "ERROR:Failed to create \"$XPCA_DTS_PKG_DIR\" "
            return 1
         fi
      fi

   if [[ ! -d $XPCA_DTS_PKG_DIR ]]
   then
      echo "ERROR:  \"$XPCA_DTS_PKG_DIR\" directory does not exist"
      to_exit=1
      return $to_exit
   fi

   to_exit=0

   return $to_exit
   
}

##################################################################################
# This function returns the list of nodes part of current package 
#
# Internal Use only
#
# Arguments: None
#
# Returns: node list
#
###############################################################################

function get_package_nodes
{
    if (( ${#SG_NODE_NAME[*]} == 1 )); then
        if [ "${SG_NODE_NAME[0]}" == "*" ]; then
            # All the cluster nodes
            package_node_names=`echo "$cmviewcl_output_mc" | awk '/^node:[^|]*\|name=/' | cut -d "=" -f 2`
        else
            # Only one node
            package_node_names=`echo "${SG_NODE_NAME[0]}"`
        fi
    else
        # Subset of cluster nodes
        package_node_names=`echo "${SG_NODE_NAME[@]}"`
    fi
    echo "$package_node_names"
}

##################################################################################
# This function returns the UUID of node name passed. 
#
# Internal Use only
#
# Arguments: node name 
#
# Returns: node list
#
###############################################################################

function get_uuid_from_node_name
{
    uuid=""
    node_name=$1
    uuid=`echo "$cmviewcl_output_mc" | grep $node_name | awk -F= '/^node:[^|]*\|uuid=/ {print $2}'`
    echo $uuid
}

##################################################################################
# This function checks whether the disks belonging to a LVM or SLVM are being
# replicated.
#
# Internal Use only  
#
#
# Arguments: None
#
#
# Returns: None 
#
###############################################################################
function lvm_disks_replicated
{

    for vg in ${list_of_vgs[@]} ; do
       # For each VG retrieve the device info from vgdisplay command and
       # add to the list of devices
        unset xpca_list_of_devices
        for device in `$PVDISPLAY 2>> /dev/null | awk -v vg=$vg '/VG Name/ { if ($3 == vg ) print pline}{pline=$3}'`
        do
            checkifmodused "dts/xpca"
            if [ $? -eq 0 ]; then
                xpca_list_of_devices[${#xpca_list_of_devices[@]}]=$device
            fi
        done
        checkifmodused "dts/xpca"
        if [ $? -eq 0 ]; then
            xpca_get_disk_info ${vg} "lvm"
        fi
    done
}

 ##################################################################################
 # This function checks whether the disk passed as an argument is being replicated
 # by XP.
 #
 # Internal Use only
 #
 #
 # Arguments: Disk Path , LVM groupname, lvm
 #
 #
 # Returns:  None
 #
 ###############################################################################
 function xpca_get_disk_info 
 {
    vg=$1
    if [[ "$2" = "lvm" ]]; then
        type_of_dg="volume group"
    fi 
    if [[ ! -z "${xpca_list_of_devices[*]}" ]];then
      if [[ $DISTRO == "REDHAT" ]]; then 
         cmd_output=`/usr/local/cmcluster/toolkit/SGCA/XPCAGetDiskInfo.pl $XPCA_DEVICE_GROUP ${xpca_list_of_devices[@]}`
      elif [[ $DISTRO == "SUSE" ]]; then
         cmd_output=`/opt/cmcluster/toolkit/SGCA/XPCAGetDiskInfo.pl $XPCA_DEVICE_GROUP ${xpca_list_of_devices[@]}`
      fi

      retVal=$?
      if [[ $retVal -eq 0 ]]; then
        cmd_output=`echo $cmd_output`
        if [[ "$cmd_output" = "$XPCA_DEVICE_GROUP" ]];then
          echo "All devices in $vg $type_of_dg are  part of a replication group $cmd_output"
          return 0
        else
          errors[$errcount]="Error: All devices in $vg $type_of_dg are a part of a replication group $cmd_output"
          (( errcount=errcount+1 ))
           errors[$errcount]="The DEVICE_GROUP parameter indicated the device group is $XPCA_DEVICE_GROUP"
          (( errcount=errcount+1 ))
          errors[$errcount]="The configured device group does not match with the real device group "
          (( errcount=errcount+1 ))
          return 1
        fi
    
       fi

       if [[ $retVal -eq 1 ]]; then
         warnings[$warncount]="Warning:The following devices in $vg $type_of_dg do not belong to any device group"
         (( warncount=warncount+1 ))
         warnings[$warncount]=`echo $cmd_output`
         (( warncount=warncount+1 ))
       fi
 
       if [[ $retVal -eq 2 ]];then
         errors[$errcount]="Error: One or more disks in $vg $type_of_dg belong to a different device group than one provided in the configuration"
         (( errcount=errcount+1 ))
         errors[$errcount]="The device group name provided in the configuration $XPCA_DEVICE_GROUP"
         (( errcount=errcount+1 ))
         errors[$errcount]=`echo $cmd_output`
         (( errcount=errcount+1 ))
       fi
 
       if [[ $retVal -eq 3 ]];then
         warnings[$warncount]="Warning: One or more disks in $vg $type_of_dg are not being replicated "
        (( warncount=warncount+1 ))
         warnings[$warncount]=`echo $cmd_output`
        (( warncount=warncount+1 ))
       fi

      if [[ $retVal -eq 4 ]];then
         errors[$errcount]="Error: The validation failed with the following message: "
         (( errcount=errcount+1 ))
         errors[$errcount]=`echo $cmd_output`
         (( errcount=errcount+1 ))
       fi
 

   fi 

 }

###############################################################################
# This function checks whether ccxxx module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - ccxxx module is used
#          1 - ccxxx module is not used
#
###############################################################################

function check_cc_module
{
        let exit_val=1
        typeset module_name;
        typeset module;

        module_name=`echo "${SG_MODULE_NAME[@]}"`

        for module in $module_name
        do
            if [[ "$module" = "dts/ccxpca" ]]; then
                let exit_val=0
                break
            fi
        done

    return $exit_val
}
