#######################################################################
#!/usr/bin/perl
# Easy deployment code for Metrocluster XPCA
# (C) Copyright 2008-2014 Hewlett-Packard Development Company, L.P.
# @(#) Metrocluster XPCA Module  Script.
# @(#) Product Name                :  Serviceguard Metrocluster 
# @(#) Product Version             :  B.12.00.00
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

package XP_ED;    # The module for XPCA parameters Easy Deployment
our $error     = 0;     # Package globals
our $error_map = {};    # An hash to store the error messages indexed by $error
our $ldev_map = {};
####################### globals for this package block ########################

my $xp_param_map = {};    # An empty hash ref to hold XPCA params & values
our $device_group_map = {};    # An empty hash ref to hold device group params alone
my $exit_status;
my $workingInstance;
my (@replicated_disks);
my (@non_replicated_disks);
my @instance_nos;


################# End of Globals #####################################

#######################################################################
#
# Function Name: do_parameter_discovery
# Input Parameters: An array of names of physical disks
# Return value: A hash reference that has all params and values
# Interface type: Public
# Description: This routine is called by the module consumers to fillin the
#		parameter->value hash map and return it
######################################################################

sub do_parameter_discovery {
    my (@disk) = @_ ;
    my $ret_val;
    $error = 0;    #set error to 0
    @replicated_disks = ();
    @non_replicated_disks = ();
    $ret_val = insert_into_map_device_group_parameters(@disk);
    if ( $ret_val eq -1 ) {
        return $xp_param_map;    # returning empty hash
    }
    insert_into_map_auto_parameters();
    if( $#non_replicated_disks == $#disk ) {
    $error  = 10;
    $error_map->{$error} = " No disks are replicated";
    }

    return ($xp_param_map,$device_group_map);

}

#################### End of Routine ##################################################

#############################################################################################
#
# Function Name: get_replicated_and_non_replicated_disks
# Input Parameter: an array of names of physical disks
# Return Value: two array references;First reference for replicated disks and second for
#		non-replicated disks
# Interface Type: Public
# Description: This rotuine will be called by module consumers to get the list of
#		of replicated and non-replicated disks
#############################################################################################

sub get_replicated_and_non_replicated_disks {
    my (@disk) = @_;
    my $ret_val;
    $error                = 0;     # set error to 0
    @non_replicated_disks = ();    # reset the global array
    @replicated_disks     = ();    # reset the global array

    $ret_val = insert_into_map_device_group_parameters(@disk);
    if ( $ret_val eq -1 ) {
        return ( \@replicated_disks, \@non_replicated_disks );
    }

    return ( \@replicated_disks, \@non_replicated_disks );

}

############### End of rotuine #########################################################

######################## End of rotuine ###############################################
sub start_raid_manager_instance {
my $instance_no = $_[0];
$error = 0;
my $exit_status = 0;
    $start_cmd = "/usr/bin/horcmstart.sh ".$instance_no;
    `$start_cmd`;
    $exit_status = $? >> 8;
    if ( ( $exit_status ne 1 ) and ( $exit_status ne 0 ) ) {
        $error = 4;
        $error_map->{$error} =
          "RaidManager instance $instance_no failed to start";
        return 1;
    }
    return 0;
}
########################################################################
#
#  Function name : get_all_horcm_files
#  Input parameters : None
#  Return Value : array of all horcm instance configuration files in /etc
#
########################################################################

sub get_all_horcm_files {

my $ls_path;
my $os_type = (uname)[0];
        if ($os_type eq "HP-UX") {
    $ls_path = "/usr/bin/ls";
} else {
    $ls_path = "/bin/ls";
}

    @horcm_files = `$ls_path /etc/horcm*.conf 2> /dev/null`;
    $exit_status = $? >> 8;
    $error       = 0;

    if ( $exit_status eq 2 ) {
        $error = 1;
        $error_map->{$error} = " RaidManager instance files not found in /etc ";
    }
    elsif ( $exit_status ne 0 ) {
        $error = 2;
        $error_map->{$error} =
          "ls command to find RaidManager instance files returned $exit_status";
    }
    return @horcm_files;
}

#################### End of routine ####################################

########################################################################
#
# Function name: get_atleast_one_horcm_instance_number
# Input parameters: array of horcm.conf file paths
# Return value :any horcm instance number on success
#               : -1 on failure
# Description : This function finds all files with the name horcm[0-9]
#		It then takes the first file and parses its name to
#		get the horcm instance number
#
#######################################################################

sub get_atleast_one_horcm_instance_number {

    my (@horcm_files) = @_;
    my $instance_filename;
    my $horcmStrIndex;
    my $dotCharIndex;
    my $found_file = 0;
    $error = 0;

    @horcm_files = get_all_horcm_files();

    foreach (@horcm_files) {
        if ( $_ =~ m/horcm[0-9]+\./g ) {
            $instance_filename = $_;
	$horcmStrIndex = index( $instance_filename, "horcm" );
        $instance_no = substr( $instance_filename, $horcmStrIndex + 5 );
        $dotCharIndex = index( $instance_no, "." );
        $instance_no = substr( $instance_no, 0, $dotCharIndex );
        push(@instance_nos,$instance_no);

        }
    }

    if (!(@instance_nos)) {
        $error = 3;
        $error_map->{$error} =
          "No files with name pattern horcm[0-9]+\\. was found";
    }
}

###################### End of routine ####################################

##########################################################################
#
# Function name: insert_into_map_device_group_parameters
# Input Parameters : array of physical disks
# Return value : 0 on success
#		 -1 on failure
# Description : This function will use raidscan command to get the device group
#		name and the fence level and the status of the device group
#
##########################################################################

sub insert_into_map_device_group_parameters {
    
    my (@disk_names) = @_;
    my (@raidscan_output);
    my $ldev_line;
    my (@number_part);
    my $device_group_line;
    my $line;
    my $ldev_num;
    my (@device_group_line_split_array);
    my (@reversed_device_group_line_split_array);
    my $device_group_name;
    my (@split_column);
    my $uniq_device_group;
    $error = 0;
    $current_mu = 0;
    get_atleast_one_horcm_instance_number();

    foreach $disk (@disk_names) {
        if ( check_and_store_pairdisplay($disk,$current_mu) eq  1 ) {
           $current_mu = $current_mu + 1;
           if ( $current_mu >  3 ) {
             push( @non_replicated_disks, $disk );
             $current_mu = 0; # reset mu
           } else {
                redo;
            }
         }
      else {
       #print $error, " : ", $error_map->{$error} , "\n";
      }
    }        
        
}

################### End of routine ##################################################

sub check_and_store_pairdisplay {
my $disk =$_[0];
my $mu = $_[1];
my @pairdisplay_output;
my @pairdisplay_split;
my $instance_no;
my $found = 1;
foreach $instance (@instance_nos) {
start_raid_manager_instance( $instance );
@pairdisplay_output = `pairdisplay -I$instance -fd -CLI -d $disk $mu -l 2>/dev/null`;
$exit_val = $? >> 8;
if( $exit_val eq 0) {
$found = 0;
$instance_no = $instance;
last;
}
}
if( $found ne 0) {
return 1;
}
shift(@pairdisplay_output);
@pairdisplay_split = split(' ',$pairdisplay_output[0]);
$device_group = trim($pairdisplay_split[0]);
$fence = trim($pairdisplay_split[8]);
$status = trim($pairdisplay_split[7]);
$state = trim($pairdisplay_split[6]);
$ldev = trim($pairdisplay_split[5]);
$ldev_map->{$disk} = $ldev;

if( $state eq "SMPL" ) {
return 1;
}else {
push( @replicated_disks, $disk);
}

 if ( !$uniq_device_group ) {
            $uniq_device_group = $device_group;
            $xp_param_map->{'DEVICE_GROUP'} = $device_group;
            $device_group_map->{'DEVICE_GROUP'} = $device_group;
            $xp_param_map->{'FENCE'} = $fence;
            $device_group_map->{'FENCE'} = $fence;
            $device_group_map->{'STATUS'} = $status;
            $device_group_map->{'STATE'} = $state;
            $device_group_map->{'MU NUMBER'} = $mu;
            $xp_param_map->{'HORCMINST'} = $instance_no;
            $device_group_map->{'HORCMINST'} = $instance_no;

        }
        elsif ( $uniq_device_group ne $device_group ) {
            $error = 7;
            $error_map->{$error} =
"Unsupported Configuration :Disks supplied are part of multiple device groups";
}
return 0;
}

#################### End of routine ####################################################

#######################################################################################
#
# Function Name: trim
# Input parameter: the string the needs to be trimmed
# Return value: the trimmed string
# Description: This function removes the leading and trailing whitespaces from the
#		string. whitespace implies [:space:] as defined by POSIX
#######################################################################################

sub trim {
    my $string = shift;
    $string =~ s/^\s+|\s+$//g;
    return $string;
}

################ End of routine #########################################################

sub insert_into_map_auto_parameters {

    $xp_param_map->{'AUTO_PSUEPSUS'}                        = 0;
    $xp_param_map->{'AUTO_FENCEDATA_SPLIT'}                 = 1;
    $xp_param_map->{'AUTO_SVOLPSUS'}                        = 0;
    $xp_param_map->{'AUTO_PSUSSSWS'}                        = 0;
    $xp_param_map->{'AUTO_SVOLPSUE'}                        = 0;
    $xp_param_map->{'AUTO_SVOLPFUS'}                        = 0;
    $xp_param_map->{'AUTO_NONCURDATA'}                      = 0;
    $xp_param_map->{'MULTIPLE_PVOL_OR_SVOL_FRAMES_FOR_PKG'} = 0;
    $xp_param_map->{'HORCMPERM'}                            = "MGRNOINST";
    $xp_param_map->{'WAITTIME'}                             = 300;
}


sub is_xpdisk {
my $started=0;
get_atleast_one_horcm_instance_number();
$error = 0;
$disk = $_[0];
my $true_instance=0;
foreach $instance (@instance_nos) {
start_raid_manager_instance( $instance );
if ( $error == 0 ) {
     $started = 1;
     $true_instance = $instance;
     last;
  } else {
     print $error, "\n";
     next;
  }
}
if( $started eq 1 ) {
`raidscan -q |raidscan -I$true_instance -find -fd -fg -pd $disk 2> /dev/null`;
 $exit_status = $? >> 8;
  if ( $exit_status eq 227 ) {
       return 1;
        }else {
       return 0;
      }
}else {
      return 1;
}

}

1;

