#!/usr/bin/perl
####################################################################
# (C) Copyright 2008-2013 Hewlett-Packard Development Company, L.P.
# @(#) Metrocluster XPCA Module  Script.
# @(#) Product Name                :  HP Metrocluster XPCA
# @(#) Product Version             :  A.11.00
# @(#) Patch Name                  :  PHSS_41663 (HP-UX 11iv2) / PHSS_41664 (HP-UX 11iv3)
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################
# This is the validation script for XPCA environment file.
# The full path to the environment file is passed in as the argument.
# ####################################################################################

$env_file = $ARGV[0];
my $fence;
my $device_group;
my $horcminst;
my $horcmperm;

extract_params_from_env_file();
validate_params();

#######################################################################################
#
# Function Name: extract_params_from_env_file
# Input parameter: none
# Return value: none
# Description: This function extracts the parameters DEVICE_GROUP, HORCMINST, FENCE,
#              HORCMPERM present in the environment file
#######################################################################################

sub extract_params_from_env_file {

    @filtered_env = `grep ^[^#] $env_file`;

    @device_group_line = grep ( /^DEVICE_GROUP/, @filtered_env );
    if ( $#device_group_line > 1 ) {
        print
"Error: Multiple DEVICE_GROUP lines found in the supplied XPCA environment file \n";
    }

    @device_group_ar = split( '=', $device_group_line[0] );
    $device_group = trim( $device_group_ar[1] );
 
      

    @horcminst_line = grep ( /^HORCMINST/, @filtered_env );
    if ( $#horcminst_line > 1 ) {
        print
"Error: Multiple HORCMINST lines found in the supplied XPCA environment file \n";
    }

    @horcminst_ar = split( '=', $horcminst_line[0] );
    $horcminst = trim( $horcminst_ar[1] );

    @fence_line = grep ( /^FENCE/, @filtered_env );
    if ( $#fence_line > 1 ) {
        print
"Error: Multiple FENCE lines found in the supplied XPCA environment file \n";
    }

    @fence_ar = split( '=', $fence_line[0] );
    $fence    = trim( $fence_ar[1] );
    $fence    = uc($fence);

    @horcmperm_line = grep ( /^HORCMPERM/, @filtered_env );
    if ( $#horcmperm_line > 1 ) {
        print
"Error: Multiple HORCMPERM lines found in the supplied XPCA environment file \n";
    }

    @horcmperm_ar = split( '=', $horcmperm_line[0] );
    $horcmperm    = trim( $horcmperm_ar[1] );
    $horcmperm    = uc($horcmperm);

}

#######################################################################################
#
# Function Name: validate_params
# Input parameter: none
# Return value: none
# Description: This function validates the parameters DEVICE_GROUP, HORCMINST, FENCE, 
#              HORCMPERM present in the environment file
#######################################################################################

sub validate_params {
    print "Starting validation of XPCA parameters \n";
    `horcmstart.sh $horcminst`;
    $exit_val = $? >> 8;
    if ( $exit_val ne 1 ) {
        if ( $exit_val ne 0 ) {
            print "Error: Validation failed with the following message \n";
            print "Error: Horcm Instance $horcminst failed to start \n";
            exit 1;
        }
    }

    @pairdisplay_output = `pairdisplay -I$horcminst -CLI -g $device_group -l`;
    $exit_val           = $? >> 8;
    if ( $exit_val ne 0 ) {
        print "Error:Validation failed with the above message \n";
        print @pairdisplay_output, "\n";
        exit 1;
    }

    shift(@pairdisplay_output);
    @pairdisplay_split = split( ' ', $pairdisplay_output[0] );
    $actual_fence      = $pairdisplay_split[10];
    $actual_fence      = trim($actual_fence);

    if ( $fence ne $actual_fence ) {
        print "Error: Validation failed with the following message \n";
        print
"Error: The FENCE $fence in the XPCA Environment file doesnt match the actual FENCE $actual_fence of Device group $device_group\n";
        exit 1;
    }

    if ( $horcmperm ne MGRNOINST ) {
        if ( -e $horcmperm ) {

            # do nothing the file exists
        }
        else {
            print "Error:Validation failed with the following message \n";
            print "Error: The $horcmperm file does not exist \n ";
        }
    }
    print "Completed validation of XPCA parameters \n";

}

#######################################################################################
#
# Function Name: trim
# Input parameter: the string the needs to be trimmed
# Return value: the trimmed string
# Description: This function removes the leading and trailing whitespaces from the
#               string. whitespace implies [:space:] as defined by POSIX
#######################################################################################

sub trim {
    my $string = shift;
    $string =~ s/^\s+|\s+$//g;
    return $string;
}

